--creating tables
CREATE TABLE [dbo].[hdCategories] (
	[CategoryID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] nvarchar (50) NOT NULL,
	[SectionID] int NULL,
	[InstanceID] int NOT NULL DEFAULT(0),
	[ForTechsOnly] bit NOT NULL DEFAULT (0),
	OrderByNumber int NOT NULL DEFAULT 1
)
GO

CREATE TABLE [dbo].[hdSections] (
	[SectionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] nvarchar (50) NOT NULL,
	[InstanceID] int NOT NULL DEFAULT(0),
	OrderByNumber int NOT NULL DEFAULT 1
)
GO

CREATE TABLE [dbo].[hdInstances] (
	[InstanceID] int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] varchar (100) NOT NULL,
	[CustomerID] int NOT NULL,
	ValidTill datetime NOT NULL,
	CustomDomain varchar(100) NULL
)
GO

CREATE TABLE [dbo].[hdCategoryPermissions] (
	[UserID] [int] NOT NULL ,
	[CategoryID] [int] NOT NULL 
)
GO
--now add complex primary key in [hdCategoryPermissions] table
ALTER TABLE [dbo].[hdCategoryPermissions] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdCategoryPermissions] PRIMARY KEY CLUSTERED 
	(
		[UserID],
		[CategoryID]
	)
GO

CREATE TABLE [dbo].[hdComments] (
	[CommentID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[IssueID] [int] NOT NULL ,
	[CommentDate] [datetime] NOT NULL DEFAULT (getdate()),
	[UserID] [int] NULL ,
	[Body] nvarchar(max) NOT NULL ,
	[ForTechsOnly] bit NOT NULL DEFAULT (0),
	[IsSystem] bit NOT NULL DEFAULT(0),
	HiddenFromKB bit NOT NULL DEFAULT(0)
)
GO

CREATE TABLE [dbo].[hdCommentTemplates] (
	[TemplateID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[Body] nvarchar(max) NOT NULL ,
)
GO

CREATE TABLE [dbo].[hdIssues] (
	[IssueID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[IssueDate] [datetime] NOT NULL DEFAULT (getdate()),
	[UserID] [int] NOT NULL ,
	[CategoryID] [int] NOT NULL ,
	[StartDate] datetime null,
	[ResolvedDate] [datetime] NULL ,
	[AssignedToUserID] [int] NULL ,
	[Subject] [nvarchar] (100) NOT NULL ,
	[Body] nvarchar(max) NOT NULL ,
	[UpdatedByUser] [bit] NOT NULL DEFAULT (0),
	[UpdatedByPerformer] [bit] NOT NULL DEFAULT (0),
	[StatusID] [int] NOT NULL DEFAULT (1),
	[PublishToKB] [bit] NOT NULL DEFAULT (0),
	[Priority] smallint NOT NULL DEFAULT (0),
	[TimeSpentInSeconds] int NOT NULL DEFAULT(0),
	[KBForTechsOnly] [bit] NOT NULL DEFAULT (0),
	[LastUpdated] datetime NOT NULL DEFAULT(getdate()),
	[LastEscalated] datetime NULL,
	EscalationsCount int NOT NULL DEFAULT(0),
	EscalationsDisabled bit NOT NULL DEFAULT(0),
	DueDate datetime NULL
)
GO

CREATE TABLE hdMergedIssues (
	[IssueID] int NOT NULL,
	[IssueMergedID] int NOT NULL
)
ALTER TABLE hdMergedIssues WITH NOCHECK ADD CONSTRAINT [PK_hdMergedIssues] PRIMARY KEY CLUSTERED (
	IssueID,
	IssueMergedID
)
GO

CREATE TABLE [dbo].[hdCustomFields] (
	[FieldID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] [int] NOT NULL,
	[Type] [int] NOT NULL,
	[FieldName] nvarchar(100) NOT NULL,
	[ForTechsOnly] bit NOT NULL DEFAULT (0),
	[Mandatory] bit NOT NULL DEFAULT (0),
	[OrderByNumber] int NOT NULL DEFAULT (100)
)
GO

CREATE TABLE hdIssueSchedule (
	IssueID int NOT NULL PRIMARY KEY,
	FreqType int NOT NULL,
	FirstRecurrenceDate datetime NOT NULL,
	PrevRepeatsCount int NOT NULL DEFAULT(0)
)
GO

CREATE TABLE hdScheduledIssues (
	ScheduledIssueID int NOT NULL PRIMARY KEY,
	OriginalIssueID int NOT NULL
)
GO

CREATE TABLE [dbo].[hdCustomFieldValues] (
	[IssueID] [int] NOT NULL,
	[FieldID] [int] NOT NULL,
	[Value] sql_variant NOT NULL
)
GO
ALTER TABLE [dbo].[hdCustomFieldValues] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdCustomFieldValues]PRIMARY KEY CLUSTERED 
	(
		[IssueID],
		[FieldID]
	)
GO

/* history table */
CREATE TABLE hdIssueViewHistory (
	UserID int NOT NULL,
	IssueID int NOT NULL,
	ViewDate datetime NOT NULL DEFAULT (getdate())
)
ALTER TABLE hdIssueViewHistory WITH NOCHECK ADD CONSTRAINT [PK_hdIssueViewHistory] PRIMARY KEY CLUSTERED (
	UserID,
	IssueID
)


CREATE TABLE [dbo].[hdCustomFieldOptions] (
	[OptionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[FieldID] [int] NOT NULL,
	[OptionValue] nvarchar(500) NOT NULL
)
GO

CREATE TABLE [dbo].[hdNews] (
	[NewsID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[NewsDate] [datetime] NOT NULL DEFAULT (getdate()),
	[UserID] [int] NOT NULL ,
	[Subject] [nvarchar] (100) NOT NULL ,
	[Body] nvarchar(max) NOT NULL
)
GO

CREATE TABLE [dbo].[hdSettings] (
	[InstanceID] int NOT NULL DEFAULT(0),
	[EmailNotificationsEnabled] [bit] NOT NULL DEFAULT (0),
	[SMTPServer] varchar (50) NULL ,
	[SMTPAuthRequired] [bit] NOT NULL  DEFAULT (0),
	[SMTPUserName] varchar (50) NULL ,
	[SMTPPassword] varchar (50) NULL ,
	[SMTPUseSSL] bit NOT NULL  DEFAULT (0) ,
	[FromAddress] varchar (50) NULL ,
	ReplyToAddress varchar (50) NULL,
	FromName varchar(50) NULL,
	UseFromNameForUserOriginatedMsgs bit not null DEFAULT 0,
	AllowUserRegistration bit NOT NULL  DEFAULT (1),
	SMTPPort int NOT NULL DEFAULT (25),
	MailCheckerEnabled bit NOT NULL DEFAULT (0),
	MailCheckerInterval int NOT NULL DEFAULT (600),
	MailCheckerNewIssuesCategoryID int NOT NULL DEFAULT (1),
	MailCheckerAllowUnregisteredUsers bit NOT NULL DEFAULT (0),
	RestrictIssueDeletion bit NOT NULL DEFAULT (0),
	Lang varchar (10) NOT NULL DEFAULT ('en-US'),
	MaxAttachSize int NOT NULL DEFAULT (1000),
	EmailTemplate varchar (2000) NOT NULL DEFAULT '',
	RestrictIssueClosing bit NOT NULL DEFAULT (0),
	NewIssuesDefaultCategoryID int NOT NULL DEFAULT 1,
	AllowGuestSubmission bit default 1,
	AllowAnonymousKBAccess bit default 1,
	EmailSubjectTemplate varchar (1000) NOT NULL DEFAULT '',
	SeeEveryonesIssues bit default 0,
	UseDefaultSmtp bit default 0,
	HeaderBGColor char(6) NOT NULL DEFAULT('000000'),
	MenuBarBGColor char(6) NOT NULL DEFAULT('FF7700'),
	MenuBarTabColor char(6) NOT NULL DEFAULT('D5783A'),
	LogoImage [image] NULL,
	SendTechniciansEmailOnNewIssue bit NOT NULL DEFAULT(1),
	ServerTimeOffset int NOT NULL DEFAULT(0),
	HoursToEscalateAfter int NOT NULL DEFAULT (0),
	DisableWelcomeMessage bit NOT NULL DEFAULT (0),
	SendHtmlEmails bit NOT NULL DEFAULT (0),
	DisableKB bit NOT NULL DEFAULT(0),
	AutoLoginSharedSecret varchar(20) NULL,
	MenuBarTextColor char(6) NOT NULL DEFAULT('FFFFFF'),
	DisableCloseNotification bit NOT NULL DEFAULT 1,
	Title varchar(500) NOT NULL DEFAULT 'Jitbit HelpDesk',
	DisableCaptcha bit not null default 0,
	DisableAssets bit not null default 0,
	DaysToCloseAfter int NOT NULL DEFAULT(0),
	DontSendEscalationToSubmitter bit NOT NULL default(0),
	CRMRootUrl varchar(1000) not null DEFAULT '',
	CRMBaseUrl varchar(1000) not null DEFAULT '',
	CRMSharedKey varchar(255) not null DEFAULT ''
)
GO

CREATE TABLE hdPOPServers (
	ServerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	HostName varchar (200) NULL,
	Port int NOT NULL DEFAULT (110),
	POPLogin varchar (100) NULL,
	POPPassword varchar (100) NULL,
	UseSSL bit NOT NULL DEFAULT (0),
	NewTicketsCategoryID int NULL,
	IsImap bit NOT NULL DEFAULT 0
)

CREATE TABLE [dbo].[hdStatus] (
	[StatusID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NULL,
	[Name] nvarchar (50) NOT NULL,
	[ButtonCaption] nvarchar(50) NULL
)
GO

CREATE TABLE [dbo].[hdUsers] (
	[UserID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[UserName] varchar (100) NOT NULL ,
	[email] varchar (100) NOT NULL ,
	[Password] varchar (50) NOT NULL ,
	[IsAdministrator] [bit] NOT NULL DEFAULT (0),
	[FirstName] nvarchar(255) NOT NULL DEFAULT (''),
	[LastName] nvarchar(255) NOT NULL DEFAULT (''),
	CompanyID int NULL,
	[Notes] varchar(4000) NOT NULL DEFAULT (''),
	[Phone] varchar(50) NULL,
	[Location] varchar(1000) NULL,
	[SendEmail] bit not null default 1,
	Disabled bit not null default 0,
	LastSeen datetime null,
	IPAddress varchar(50) null,
	HostName varchar(255) null,
	Lang varchar(10) NOT NULL DEFAULT(''),
	UserAgent varchar(255) null
)
GO
--now add constraint - unique username/instance - in hdUsers table
ALTER TABLE [dbo].[hdUsers] WITH NOCHECK ADD 
	CONSTRAINT [IX_hdUsers_UniqueNameInstance] UNIQUE  NONCLUSTERED 
	(
		[username],
		InstanceID
	)
GO

CREATE TABLE [dbo].[hdCompanies] (
	CompanyID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[InstanceID] int NOT NULL DEFAULT(0),
	[Name] nvarchar(255) NOT NULL,
	CRMID int NULL
)


CREATE TABLE [dbo].[hdIssueSubscribers] (
	[IssueID] [int] NOT NULL,
	[UserID] [int] NOT NULL
)
GO
--now add complex primary key in [hdIssueSubscribers] table
ALTER TABLE [dbo].[hdIssueSubscribers] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdIssueSubscribers] PRIMARY KEY CLUSTERED 
	(
		[IssueID],
		[UserID]
	)
GO

CREATE TABLE [dbo].[hdFileAttachments](
		[FileID] [int] IDENTITY(1,1) NOT NULL PRIMARY KEY,
		[IssueID] [int] NOT NULL,
		[CommentID] int NOT NULL DEFAULT (0),
		[UserID] [int] NOT NULL,
		[FileName] nvarchar(255) NOT NULL,
		[FileData] [image] NOT NULL
)

--ticket-to-ticket links
CREATE TABLE hdLinkedIssues (
	Issue1ID int NOT NULL,
	Issue2ID int NOT NULL
)
ALTER TABLE hdLinkedIssues WITH NOCHECK ADD 
	CONSTRAINT [PK_hdLinkedIssues] PRIMARY KEY CLUSTERED 
	([Issue1ID],[Issue2ID])
GO


--asset management tables
CREATE TABLE [dbo].[hdAssets] (
	ItemID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	ModelName nvarchar(1000) NOT NULL,
	TypeID int NOT NULL,
	ManufacturerID int NOT NULL,
	Quantity int NOT NULL DEFAULT(0),
	SupplierID int NOT NULL,
	SerialNumber nvarchar(200) NULL,
	Location nvarchar(500) NULL,
	Comments nvarchar(1000) NULL
)
CREATE TABLE [dbo].[hdAssetManufacturers] (
	ManufacturerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	[Name] nvarchar(1000) NOT NULL
)
CREATE TABLE [dbo].[hdAssetSuppliers] (
	SupplierID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	[Name] nvarchar(1000) NOT NULL
)
CREATE TABLE [dbo].[hdAssetTypes] (
	TypeID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	InstanceID int NOT NULL,
	[Name] nvarchar(1000) NOT NULL
)
CREATE TABLE [dbo].[hdAssetUsers] (
	ItemID int NOT NULL,
	UserID int NOT NULL
)
ALTER TABLE [dbo].[hdAssetUsers] WITH NOCHECK ADD 
	CONSTRAINT [PK_hdAssetUsers] PRIMARY KEY CLUSTERED ([ItemID],[UserID])
GO



--creating foreign keys
ALTER TABLE [dbo].[hdCategoryPermissions] ADD 
	CONSTRAINT [FK_hdCategoryPermissions_hdCategories] FOREIGN KEY([CategoryID]) REFERENCES [dbo].[hdCategories] ([CategoryID]),
	CONSTRAINT [FK_hdCategoryPermissions_hdUsers] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

ALTER TABLE [dbo].[hdComments] ADD 
	CONSTRAINT [FK_hdComments_hdIssues] FOREIGN KEY([IssueID]) REFERENCES [dbo].[hdIssues] ([IssueID]),
	CONSTRAINT [FK_hdComments_hdUsers] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

ALTER TABLE [dbo].[hdIssues] ADD 
	CONSTRAINT [FK_hdIssues_hdCategories] FOREIGN KEY([CategoryID]) REFERENCES [dbo].[hdCategories] ([CategoryID]),
	CONSTRAINT [FK_hdIssues_hdStatus] FOREIGN KEY([StatusID]) REFERENCES [dbo].[hdStatus] ([StatusID]),
	CONSTRAINT [FK_hdIssues_hdUsers1] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID]),
	CONSTRAINT [FK_hdIssues_hdUsers3] FOREIGN KEY([AssignedToUserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

ALTER TABLE [dbo].[hdFileAttachments] ADD
	CONSTRAINT [FK_hdFileAttachments_hdIssues] FOREIGN KEY([IssueID]) REFERENCES [dbo].[hdIssues] ([IssueID]),
	CONSTRAINT [FK_hdFileAttachments_hdUsers] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

ALTER TABLE [dbo].[hdIssueSubscribers] ADD
	CONSTRAINT [FK_hdIssueSubscribers_hdIssues] FOREIGN KEY([IssueID]) REFERENCES [dbo].[hdIssues] ([IssueID]),
	CONSTRAINT [FK_hdIssueSubscribers_hdUsers] FOREIGN KEY([UserID]) REFERENCES [dbo].[hdUsers] ([UserID])
GO

--indexes
CREATE INDEX IX_hdComments_IssueID ON hdComments (IssueID)
CREATE INDEX IX_hdIssues_CategoryID ON hdIssues (CategoryID)
CREATE INDEX IX_hdIssues_UserID ON hdIssues (UserID)

--create TWO PRE-DEFINED USERS
INSERT INTO hdUsers (username, email, password, IsAdministrator)
VALUES ('admin', 'admin@testmail', 'admin', 1)
GO

INSERT INTO hdUsers (username, email, password, IsAdministrator)
VALUES ('client', 'client@testmail', 'client', 0)
GO

--create pre-defined sections
INSERT INTO hdSections (Name) VALUES ('Sales')
go

INSERT INTO hdSections (Name) VALUES ('Tech')
go

--create PRE-DEFINED CATEGORIES
INSERT INTO hdCategories (Name, SectionID) VALUES ('General sales', 1)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Hardware', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Software', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Network & Internet', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Telephony', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Printers & Copiers', 2)
go

INSERT INTO hdCategories (Name, SectionID) VALUES ('Accounting', 1)
go

--create default STATUSes DO NOT REMOVE!!!!!!!!!!!!!!!!
INSERT INTO hdStatus (Name) VALUES ('New')
go

INSERT INTO hdStatus (Name) VALUES ('In progress')
go

INSERT INTO hdStatus (Name) VALUES ('Resolved')
go
