--the script checks the database format
--and updates it to the latest version if necessary
--run it against the Helpdesk's database to upgrade it!
--PS. most recent changes are at the bottom... top-to-bottom order.


--###########################################################################

--fileAttachments table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdFileAttachments]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdFileAttachments](
		[FileID] [int] IDENTITY(1,1) PRIMARY KEY NOT NULL,
		[IssueID] [int] NOT NULL,
		[UserID] [int] NOT NULL,
		[FileName] [varchar](255) NOT NULL,
		[FileData] [image] NOT NULL,
	)
END
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdFileAttachments_hdIssues]'))
ALTER TABLE hdFileAttachments WITH CHECK ADD CONSTRAINT [FK_hdFileAttachments_hdIssues] FOREIGN KEY([IssueID])
REFERENCES hdIssues (IssueID)
GO
ALTER TABLE hdFileAttachments CHECK CONSTRAINT [FK_hdFileAttachments_hdIssues]
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdFileAttachments_hdUsers]'))
ALTER TABLE hdFileAttachments WITH CHECK ADD CONSTRAINT [FK_hdFileAttachments_hdUsers] FOREIGN KEY([UserID])
REFERENCES hdUsers (UserID)
GO
ALTER TABLE hdFileAttachments CHECK CONSTRAINT [FK_hdFileAttachments_hdUsers]
GO
--fileattachmetns table finished

--allowUserRegistration column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowUserRegistration')
begin
	  ALTER TABLE hdSettings ADD AllowUserRegistration bit NOT NULL DEFAULT 1
end

--###########add MailChecker columns
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerEnabled')
begin
	  ALTER TABLE hdSettings ADD MailCheckerEnabled bit NOT NULL DEFAULT 0
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerInterval')
begin
	  ALTER TABLE hdSettings ADD MailCheckerInterval int NOT NULL DEFAULT 600
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerNewIssuesCategoryID')
begin
	  ALTER TABLE hdSettings ADD MailCheckerNewIssuesCategoryID int NOT NULL DEFAULT (1)
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerAllowUnregisteredUsers')
begin
	  ALTER TABLE hdSettings ADD MailCheckerAllowUnregisteredUsers bit NOT NULL DEFAULT (0)
end
--#############

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='Priority')
begin
	  ALTER TABLE hdIssues ADD Priority smallint NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPPort')
begin
	  ALTER TABLE hdSettings ADD SMTPPort int NOT NULL DEFAULT 25
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPUseSSL')
begin
	  ALTER TABLE hdSettings ADD SMTPUseSSL bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictIssueDeletion')
begin
	  ALTER TABLE hdSettings ADD RestrictIssueDeletion bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MaxAttachSize')
begin
	  ALTER TABLE hdSettings ADD MaxAttachSize int NOT NULL DEFAULT (1000)
end


if not exists (select * from syscolumns where id=object_id('hdSettings') and name='Lang')
begin
	  ALTER TABLE hdSettings ADD Lang [varchar] (10) NOT NULL DEFAULT ('en-US')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='FirstName')
begin
	  ALTER TABLE hdUsers ADD FirstName varchar(255) NOT NULL DEFAULT ('')
end
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastName')
begin
	  ALTER TABLE hdUsers ADD LastName varchar(255) NOT NULL DEFAULT ('')
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictIssueClosing')
begin
	  ALTER TABLE hdSettings ADD RestrictIssueClosing bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='EmailTemplate')
begin
	  ALTER TABLE hdSettings ADD EmailTemplate varchar(2000) NOT NULL DEFAULT ('')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Notes')
begin
	  ALTER TABLE hdUsers ADD Notes varchar(4000) NOT NULL DEFAULT ('')
end

--Sections table		  
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdSections]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdSections] (
	[SectionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] [varchar] (50) NOT NULL
	)
END

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='SectionID')
begin
	  ALTER TABLE hdCategories ADD SectionID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NewIssuesDefaultCategoryID')
begin
	  ALTER TABLE hdSettings ADD NewIssuesDefaultCategoryID int NOT NULL DEFAULT 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Location')
begin
	  ALTER TABLE hdUsers ADD Location varchar(1000) NULL
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Phone')
begin
	  ALTER TABLE hdUsers ADD Phone varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='KBForTechsOnly')
begin
	  ALTER TABLE hdIssues ADD KBForTechsOnly bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='ForTechsOnly')
begin
	  ALTER TABLE hdComments ADD ForTechsOnly bit NOT NULL DEFAULT 0
end


/*#####!!!multiinstance changes start #################*/
--InstanceID column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='InstanceID')
begin
	  ALTER TABLE hdSettings ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdCategories] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdSections] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdIssues] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdUsers] ADD InstanceID int NOT NULL DEFAULT 0
end
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdInstances]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdInstances] (
		[InstanceID] int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[Name] [varchar] (100) NOT NULL,
		[CustomerID] int NOT NULL,
		ValidTill datetime NOT NULL
	)
END
GO

--drop unique user name constraint
if exists (select * from sysobjects where name = 'IX_hdUsers_UniqueName')
begin
	alter table hdUsers drop constraint IX_hdUsers_UniqueName
	
	--add username/instanceid uniqeness
	if not exists (select * from sysobjects where name = 'IX_hdUsers_UniqueNameInstance')
	begin
		ALTER TABLE [dbo].[hdUsers] WITH NOCHECK ADD 
		CONSTRAINT [IX_hdUsers_UniqueNameInstance] UNIQUE  NONCLUSTERED 
		(
			[username],
			InstanceID
		)
	end
end
/*##################!!!!!!!!!!end of multiinstance code*/
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='UseDefaultSmtp')
begin
	  ALTER TABLE hdSettings ADD UseDefaultSmtp bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowAnonymousKBAccess')
begin
	  ALTER TABLE hdSettings ADD AllowAnonymousKBAccess bit not null default 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowGuestSubmission')
begin
	  ALTER TABLE hdSettings ADD AllowGuestSubmission bit not null default 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='SendEmail')
begin
	  ALTER TABLE hdUsers ADD SendEmail bit not null default 1
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HeaderBGColor')
begin
	  ALTER TABLE hdSettings ADD HeaderBGColor char(6) NOT NULL DEFAULT('000000')
	  ALTER TABLE hdSettings ADD MenuBarBGColor char(6) NOT NULL DEFAULT('FF7700')
	  ALTER TABLE hdSettings ADD LogoImage [image] NULL
end
GO
	
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SeeEveryonesIssues')
begin
	  ALTER TABLE hdSettings ADD SeeEveryonesIssues bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='EmailSubjectTemplate')
begin
	  ALTER TABLE hdSettings ADD EmailSubjectTemplate varchar (1000) NOT NULL DEFAULT ''
end
GO

--enlarge the password field so MD5-string fits there
alter table hdUsers alter column Password [varchar](50) NOT NULL

--dropping IX_hdUsers_UniqueEmailInstance constraint - not needed.
if exists (select * from sysobjects where name = 'IX_hdUsers_UniqueEmailInstance')
begin
	alter table hdUsers drop constraint IX_hdUsers_UniqueEmailInstance
end
GO

--converting Body and Subject and Status from varchar to Nvarchar
alter table hdIssues alter column Subject nvarchar(100) NOT NULL
alter table hdStatus alter column [Name] nvarchar (50) NOT NULL
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendTechniciansEmailOnNewIssue')
begin
	ALTER TABLE hdSettings ADD SendTechniciansEmailOnNewIssue bit NOT NULL DEFAULT 1
end

--adding hdNews table
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdNews')
begin
	CREATE TABLE [dbo].[hdNews] (
		[NewsID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[NewsDate] [datetime] NOT NULL DEFAULT (getdate()),
		[UserID] [int] NOT NULL ,
		[Subject] [nvarchar] (100) NOT NULL ,
		[Body] [nvarchar] (4000) NOT NULL
	)
end
GO

--adding "Custom Fields" functionality
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdCustomFields')
begin
	CREATE TABLE [dbo].[hdCustomFields] (
		[FieldID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] [int] NOT NULL,
		[Type] [int] NOT NULL,
		[FieldName] nvarchar(100) NOT NULL
	)
	CREATE TABLE [dbo].[hdCustomFieldValues] (
		[IssueID] [int] NOT NULL,
		[FieldID] [int] NOT NULL,
		[Value] sql_variant NOT NULL
	)
	ALTER TABLE [dbo].[hdCustomFieldValues] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCustomFieldValues]PRIMARY KEY CLUSTERED 
		(
			[IssueID],
			[FieldID]
		)
	CREATE TABLE [dbo].[hdCustomFieldOptions] (
		[OptionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[FieldID] [int] NOT NULL,
		[OptionValue] nvarchar(500) NOT NULL
	)
end
GO


if not exists (select * from syscolumns where id=object_id('hdNews') and name='InstanceID')
begin
	ALTER TABLE hdNews ADD InstanceID int NOT NULL DEFAULT(0)
end

--enlarging username and email columns
alter table hdUsers alter column UserName varchar(100) NOT NULL
alter table hdUsers alter column email varchar(100) NOT NULL

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ServerTimeOffset')
begin
	ALTER TABLE hdSettings ADD ServerTimeOffset int NOT NULL DEFAULT 0
end

--adding index to hdComments (IssueID
if not exists (select name FROM sysindexes WHERE name = 'IX_hdComments_IssueID') begin
	CREATE INDEX IX_hdComments_IssueID ON hdComments (IssueID)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ReplyToAddress')
begin
	ALTER TABLE hdSettings ADD ReplyToAddress varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='StartDate')
begin
	ALTER TABLE hdIssues ADD StartDate datetime NULL
end

--hdIssueSubscribers table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdIssueSubscribers]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdIssueSubscribers] (
		[IssueID] [int] NOT NULL,
		[UserID] [int] NOT NULL
	)
END
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdIssueSubscribers_hdIssues]'))
ALTER TABLE hdIssueSubscribers WITH CHECK ADD CONSTRAINT [FK_hdIssueSubscribers_hdIssues] FOREIGN KEY([IssueID])
REFERENCES hdIssues (IssueID)
GO
ALTER TABLE hdIssueSubscribers CHECK CONSTRAINT [FK_hdIssueSubscribers_hdIssues]
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdIssueSubscribers_hdUsers]'))
ALTER TABLE hdIssueSubscribers WITH CHECK ADD CONSTRAINT [FK_hdIssueSubscribers_hdUsers] FOREIGN KEY([UserID])
REFERENCES hdUsers (UserID)
GO
ALTER TABLE hdIssueSubscribers CHECK CONSTRAINT [FK_hdIssueSubscribers_hdUsers]
GO
--hdIssueSubscribers table finished

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HoursToEscalateAfter')
begin
	  ALTER TABLE hdSettings ADD HoursToEscalateAfter int NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='LastUpdated')
begin
	  ALTER TABLE hdIssues ADD LastUpdated datetime NOT NULL DEFAULT(getdate())
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='LastEscalated')
begin
	  ALTER TABLE hdIssues ADD LastEscalated datetime NULL
end

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='ButtonCaption')
begin
	  ALTER TABLE hdStatus ADD ButtonCaption nvarchar(50) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='InstanceID')
begin
	  ALTER TABLE hdStatus ADD InstanceID int NULL
end

--converting from nvarchar(4000) to Nvarchar(MAX)
alter table hdNews alter column Body nvarchar(MAX) NOT NULL
alter table hdIssues alter column Body nvarchar(MAX) NOT NULL
alter table hdComments alter column Body nvarchar(MAX) NOT NULL
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableWelcomeMessage')
begin
	  ALTER TABLE hdSettings ADD DisableWelcomeMessage bit NOT NULL DEFAULT (0)
end

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='ForTechsOnly')
begin
	  ALTER TABLE hdCustomFields ADD ForTechsOnly bit NOT NULL DEFAULT (0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MenuBarTabColor')
begin
	  ALTER TABLE hdSettings ADD MenuBarTabColor char(6) NOT NULL DEFAULT('D5783A')
end

--asset management tables
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdAssets')
begin
	CREATE TABLE [dbo].[hdAssets] (
		ItemID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		ModelName nvarchar(1000) NOT NULL,
		TypeID int NOT NULL,
		ManufacturerID int NOT NULL,
		Quantity int NOT NULL DEFAULT(0),
		SupplierID int NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetManufacturers] (
		ManufacturerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetSuppliers] (
		SupplierID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetTypes] (
		TypeID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetUsers] (
		ItemID int NOT NULL,
		UserID int NOT NULL
	)
	ALTER TABLE [dbo].[hdAssetUsers] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdAssetUsers] PRIMARY KEY CLUSTERED ([ItemID],[UserID])
end
GO

--adding [Mandatory] and [OrderByNumber] to hdCustomFields
if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='Mandatory')
begin
	  ALTER TABLE hdCustomFields ADD [Mandatory] bit NOT NULL DEFAULT (0)
	  ALTER TABLE hdCustomFields ADD [OrderByNumber] int NOT NULL DEFAULT (100)
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendHtmlEmails')
begin
	  ALTER TABLE hdSettings ADD SendHtmlEmails bit NOT NULL DEFAULT (0)
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='EscalationsCount')
begin
	  ALTER TABLE hdIssues ADD EscalationsCount int NOT NULL DEFAULT(0)
end

--adding "viewhistory" tables
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdIssueViewHistory') AND type in (N'U'))
BEGIN
	CREATE TABLE hdIssueViewHistory (
		UserID int NOT NULL,
		IssueID int NOT NULL,
		ViewDate datetime NOT NULL DEFAULT (getdate())
	)
	ALTER TABLE hdIssueViewHistory WITH NOCHECK ADD CONSTRAINT [PK_hdIssueViewHistory] PRIMARY KEY CLUSTERED (
		UserID,
		IssueID
	)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='FromName')
begin
	  ALTER TABLE hdSettings ADD FromName varchar(50) NULL
end

--more assets fields
if not exists (select * from syscolumns where id=object_id('hdAssets') and name='SerialNumber')
begin
	ALTER TABLE hdAssets ADD
		SerialNumber nvarchar(200) NULL,
		Location nvarchar(500) NULL,
		Comments nvarchar(1000) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableKB')
begin
	  ALTER TABLE hdSettings ADD DisableKB bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AutoLoginSharedSecret')
begin
	  ALTER TABLE hdSettings ADD AutoLoginSharedSecret varchar(20) NULL
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='EscalationsDisabled')
begin
	  ALTER TABLE hdIssues ADD EscalationsDisabled bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MenuBarTextColor')
begin
	  ALTER TABLE hdSettings ADD MenuBarTextColor char(6) NOT NULL DEFAULT('FFFFFF')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Disabled')
begin
	  ALTER TABLE hdUsers ADD Disabled bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableCloseNotification')
begin
	  ALTER TABLE hdSettings ADD DisableCloseNotification bit NOT NULL DEFAULT 1
end

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='ForTechsOnly')
begin
	ALTER TABLE hdCategories ADD  ForTechsOnly bit NOT NULL DEFAULT (0)
end

--adding commant teamplets table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCommentTemplates') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCommentTemplates] (
		[TemplateID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[Body] nvarchar(max) NOT NULL ,
	)
END
GO

--converting filenames to nvarchar from varchar
alter table hdFileAttachments alter column FileName nvarchar(255) NOT NULL

--adding "Companies" table and moving data to it
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCompanies') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCompanies] (
		CompanyID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[Name] nvarchar(255) NOT NULL
	)
	
	ALTER TABLE hdUsers ADD CompanyID int NULL
	
	if exists (select * from syscolumns where id=object_id('hdUsers') and name='Company')
	begin
		declare @sql nvarchar(max)
		set @sql = N'INSERT INTO hdCompanies (InstanceID, [Name])
		SELECT DISTINCT InstanceID, Company FROM hdUsers WHERE Company IS NOT NULL AND RTRIM(Company)<>'''''
		exec sp_executesql @sql
	
		set @sql = N'UPDATE hdUsers SET CompanyID = hdCompanies.CompanyID
		from hdUsers
		inner join hdCompanies ON hdUsers.Company=hdCompanies.Name
		where hdUsers.Company IS NOT NULL'
		exec sp_executesql @sql
	end
END
GO

if exists (select * from syscolumns where id=object_id('hdUsers') and name='Company')
begin
	ALTER TABLE hdUsers DROP COLUMN Company
end

--adding "POPServers" table and moving data to it
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdPOPServers') AND type in (N'U'))
BEGIN
	CREATE TABLE hdPOPServers (
		ServerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		HostName varchar (50) NULL,
		Port int NOT NULL DEFAULT (110),
		POPLogin varchar (50) NULL,
		POPPassword varchar (50) NULL,
		UseSSL bit NOT NULL DEFAULT (0)
	)
	declare @sql nvarchar(max)
	set @sql = N'INSERT INTO hdPOPServers (InstanceID, HostName, Port, POPLogin, POPPassword, UseSSL)
	SELECT InstanceID, MailCheckerPOPServer, MailCheckerPOPPort, MailCheckerPOPLogin, MailCheckerPOPPassword, MailCheckerUseSSL
	FROM hdSettings
	WHERE MailCheckerPOPServer IS NOT NULL AND MailCheckerPOPServer<>'''''
	exec sp_executesql @sql
END
GO
--removing unneeded POP cols from hdsettings
if exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerPOPServer') begin
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'MailCheckerPOPPort')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'MailCheckerUseSSL')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	ALTER TABLE hdSettings DROP COLUMN MailCheckerPOPServer, MailCheckerPOPPort, MailCheckerPOPLogin, MailCheckerPOPPassword, MailCheckerUseSSL
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='NewTicketsCategoryID')
begin
	ALTER TABLE hdPOPServers ADD NewTicketsCategoryID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='IsSystem')
begin
	ALTER TABLE hdComments ADD IsSystem bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdInstances') and name='CustomDomain')
begin
	ALTER TABLE hdInstances ADD CustomDomain varchar(100) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='Title')
begin
	ALTER TABLE hdSettings ADD Title varchar(500) NOT NULL DEFAULT ''
end
GO
--moving from hdinstances if any
if exists (select * from syscolumns where id=object_id('hdInstances') and name='Description')
begin
	declare @sql nvarchar(max)
	set @sql = N'update hdSettings SET hdSettings.Title = hdInstances.Description
	from hdInstances inner join hdSettings ON hdInstances.InstanceID=hdSettings.InstanceID'
	exec sp_executesql @sql

	ALTER TABLE hdInstances DROP COLUMN Description
end

--adding hdLinkedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdLinkedIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE hdLinkedIssues (
		Issue1ID int NOT NULL,
		Issue2ID int NOT NULL
	)
	ALTER TABLE hdLinkedIssues WITH NOCHECK ADD 
		CONSTRAINT [PK_hdLinkedIssues] PRIMARY KEY CLUSTERED 
		([Issue1ID],[Issue2ID])
END
GO

--adding hdIssueSchedule table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdIssueSchedule') AND type in (N'U'))
BEGIN
	CREATE TABLE hdIssueSchedule (
		[IssueID] int NOT NULL PRIMARY KEY,
		FreqType int NOT NULL,
		FirstRecurrenceDate datetime NOT NULL,
		PrevRepeatsCount int NOT NULL DEFAULT(0)
	)
	CREATE TABLE hdScheduledIssues (
		ScheduledIssueID int NOT NULL PRIMARY KEY,
		OriginalIssueID int NOT NULL
	)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableCaptcha')
begin
	ALTER TABLE hdSettings ADD DisableCaptcha bit not null default 0
end

--adding "OrderByNumber" to hdCategories
if not exists (select * from syscolumns where id=object_id('hdCategories') and name='OrderByNumber')
begin
	ALTER TABLE hdCategories ADD OrderByNumber int NOT NULL DEFAULT 1
	ALTER TABLE hdSections ADD OrderByNumber int NOT NULL DEFAULT 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastSeen')
begin
	ALTER TABLE hdUsers ADD LastSeen datetime null
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='HiddenFromKB')
begin
	ALTER TABLE hdComments ADD HiddenFromKB bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableAssets')
begin
	ALTER TABLE hdSettings ADD DisableAssets bit not null default 0
end

--adding hdMergedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdMergedIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE hdMergedIssues (
		[IssueID] int NOT NULL,
		[IssueMergedID] int NOT NULL
	)
	ALTER TABLE hdMergedIssues WITH NOCHECK ADD CONSTRAINT [PK_hdMergedIssues] PRIMARY KEY CLUSTERED (IssueID, IssueMergedID)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DaysToCloseAfter')
begin
	ALTER TABLE hdSettings ADD DaysToCloseAfter int NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='IPAddress')
begin
	ALTER TABLE hdUsers ADD IPAddress varchar(50) null
	ALTER TABLE hdUsers ADD HostName varchar(255) null
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Lang')
begin
	  ALTER TABLE hdUsers ADD Lang varchar(10)  NOT NULL DEFAULT('')
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='IsImap')
begin
	  ALTER TABLE hdPOPServers ADD IsImap bit NOT NULL DEFAULT 0
end

--adding indexes to hdIssues
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_CategoryID') begin
	CREATE INDEX IX_hdIssues_CategoryID ON hdIssues (CategoryID)
end
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_UserID') begin
	CREATE INDEX IX_hdIssues_UserID ON hdIssues (UserID)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='CRMBaseUrl')
begin
	  ALTER TABLE hdSettings ADD CRMRootUrl varchar(1000) not null DEFAULT ''
	  ALTER TABLE hdSettings ADD CRMBaseUrl varchar(1000) not null DEFAULT ''
	  ALTER TABLE hdSettings ADD CRMSharedKey varchar(255) not null DEFAULT ''

	  ALTER TABLE hdCompanies ADD CRMID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DontSendEscalationToSubmitter')
begin
	ALTER TABLE hdSettings ADD DontSendEscalationToSubmitter bit NOT NULL default(0)
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='UserAgent')
begin
	  ALTER TABLE hdUsers ADD UserAgent varchar(255) null
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='DueDate')
begin
	  ALTER TABLE hdIssues ADD DueDate datetime NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='UseFromNameForUserOriginatedMsgs')
begin
	ALTER TABLE hdSettings ADD UseFromNameForUserOriginatedMsgs bit NOT NULL default(0)
end

if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='CommentID')
begin
	ALTER TABLE hdFileAttachments ADD CommentID int NOT NULL DEFAULT (0)
end

--TimeSpentInSeconds col
if not exists (select * from syscolumns where id=object_id('hdIssues') and name='TimeSpentInSeconds')
begin
	ALTER TABLE hdIssues ADD TimeSpentInSeconds int NOT NULL DEFAULT (0)
end
GO
if exists (select * from syscolumns where id=object_id('hdIssues') and name='HoursSpent')
begin
	exec sp_executesql N'UPDATE hdIssues SET TimeSpentInSeconds=CAST(HoursSpent * 3600 as int)'
	
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdIssues') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdIssues') and name = 'HoursSpent')
	if not @default is null
	begin
		set @sql = N'alter table hdIssues drop constraint ' + @default
		exec sp_executesql @sql
	end
	
	ALTER TABLE hdIssues DROP COLUMN HoursSpent
end